@@ -527,7 +527,7 @@ namespace xt
     {
         static_assert(!detail::is_fixed<shape_type>::value, "Calling compute_cached_shape on fixed!");
 
-        m_cache.shape = xtl::make_sequence<xindex_type_t<inner_shape_type>>(compute_dimension(), size_type(0));
+        m_cache.shape = uninitialized_shape<xindex_type_t<inner_shape_type>>(compute_dimension());
         m_cache.is_trivial = broadcast_shape(m_cache.shape, false);
         m_cache.is_initialized = true;
     }
